from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from ..models.abuse_cases.abuse_case import AbuseCase

@dataclass
class AbusecaseGetResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The CurrentPage property
    current_page: Optional[int] = None
    # The HasMoreItems property
    has_more_items: Optional[bool] = None
    # The Items property
    items: Optional[List[AbuseCase]] = None
    # The TotalItems property
    total_items: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AbusecaseGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AbusecaseGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AbusecaseGetResponse()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from ..models.abuse_cases.abuse_case import AbuseCase

        from ..models.abuse_cases.abuse_case import AbuseCase

        fields: Dict[str, Callable[[Any], None]] = {
            "CurrentPage": lambda n : setattr(self, 'current_page', n.get_int_value()),
            "HasMoreItems": lambda n : setattr(self, 'has_more_items', n.get_bool_value()),
            "Items": lambda n : setattr(self, 'items', n.get_collection_of_object_values(AbuseCase)),
            "TotalItems": lambda n : setattr(self, 'total_items', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("CurrentPage", self.current_page)
        writer.write_bool_value("HasMoreItems", self.has_more_items)
        writer.write_collection_of_object_values("Items", self.items)
        writer.write_int_value("TotalItems", self.total_items)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class VideoLibrary_GoogleWidevineDrm(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The CertificateExpirationDate property
    certificate_expiration_date: Optional[datetime.datetime] = None
    # The CertificateId property
    certificate_id: Optional[int] = None
    # The Enabled property
    enabled: Optional[bool] = None
    # The MinClientSecurityLevel property
    min_client_security_level: Optional[float] = None
    # The Provider property
    provider: Optional[str] = None
    # The SdOnlyForL3 property
    sd_only_for_l3: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VideoLibrary_GoogleWidevineDrm:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VideoLibrary_GoogleWidevineDrm
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VideoLibrary_GoogleWidevineDrm()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "CertificateExpirationDate": lambda n : setattr(self, 'certificate_expiration_date', n.get_datetime_value()),
            "CertificateId": lambda n : setattr(self, 'certificate_id', n.get_int_value()),
            "Enabled": lambda n : setattr(self, 'enabled', n.get_bool_value()),
            "MinClientSecurityLevel": lambda n : setattr(self, 'min_client_security_level', n.get_float_value()),
            "Provider": lambda n : setattr(self, 'provider', n.get_str_value()),
            "SdOnlyForL3": lambda n : setattr(self, 'sd_only_for_l3', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("CertificateExpirationDate", self.certificate_expiration_date)
        writer.write_int_value("CertificateId", self.certificate_id)
        writer.write_bool_value("Enabled", self.enabled)
        writer.write_float_value("MinClientSecurityLevel", self.min_client_security_level)
        writer.write_str_value("Provider", self.provider)
        writer.write_bool_value("SdOnlyForL3", self.sd_only_for_l3)
        writer.write_additional_data_value(self.additional_data)
    




from enum import Enum

class Video_category(str, Enum):
    Adult = "Adult",
    AnimalsBirds = "Animals Birds",
    AnimalsCats = "Animals Cats",
    AnimalsDogs = "Animals Dogs",
    Animated = "Animated",
    Anime = "Anime",
    Gaming = "Gaming",
    Graphics = "Graphics",
    Illustrations = "Illustrations",
    Movie = "Movie",
    Other = "Other",
    OtherPeople = "Other People",
    SportsBasketball = "Sports Basketball",
    SportsHockey = "Sports Hockey",
    SportsOther = "Sports Other",
    SportsRacing = "Sports Racing",
    SportsSoccer = "Sports Soccer",
    SportsTennis = "Sports Tennis",
    Unknown = "unknown",




from __future__ import annotations
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .statistics.statistics_request_builder import StatisticsRequestBuilder

class WafRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /pullzone/{-id}/waf
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new WafRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/pullzone/{%2Did}/waf", path_parameters)
    
    @property
    def statistics(self) -> StatisticsRequestBuilder:
        """
        The statistics property
        """
        from .statistics.statistics_request_builder import StatisticsRequestBuilder

        return StatisticsRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class DnsRecord_GeolocationInfo(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The City property
    city: Optional[str] = None
    # The Country property
    country: Optional[str] = None
    # The Latitude property
    latitude: Optional[float] = None
    # The Longitude property
    longitude: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> DnsRecord_GeolocationInfo:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: DnsRecord_GeolocationInfo
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return DnsRecord_GeolocationInfo()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "City": lambda n : setattr(self, 'city', n.get_str_value()),
            "Country": lambda n : setattr(self, 'country', n.get_str_value()),
            "Latitude": lambda n : setattr(self, 'latitude', n.get_float_value()),
            "Longitude": lambda n : setattr(self, 'longitude', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("City", self.city)
        writer.write_str_value("Country", self.country)
        writer.write_float_value("Latitude", self.latitude)
        writer.write_float_value("Longitude", self.longitude)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .statistics_bandwidth_cached_chart import Statistics_BandwidthCachedChart
    from .statistics_bandwidth_used_chart import Statistics_BandwidthUsedChart
    from .statistics_cache_hit_rate_chart import Statistics_CacheHitRateChart
    from .statistics_error3xx_chart import Statistics_Error3xxChart
    from .statistics_error4xx_chart import Statistics_Error4xxChart
    from .statistics_error5xx_chart import Statistics_Error5xxChart
    from .statistics_geo_traffic_distribution import Statistics_GeoTrafficDistribution
    from .statistics_origin_response_time_chart import Statistics_OriginResponseTimeChart
    from .statistics_origin_shield_bandwidth_used_chart import Statistics_OriginShieldBandwidthUsedChart
    from .statistics_origin_shield_internal_bandwidth_used_chart import Statistics_OriginShieldInternalBandwidthUsedChart
    from .statistics_origin_traffic_chart import Statistics_OriginTrafficChart
    from .statistics_pull_requests_pulled_chart import Statistics_PullRequestsPulledChart
    from .statistics_requests_served_chart import Statistics_RequestsServedChart
    from .statistics_user_balance_history_chart import Statistics_UserBalanceHistoryChart

@dataclass
class Statistics(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The AverageOriginResponseTime property
    average_origin_response_time: Optional[int] = None
    # The BandwidthCachedChart property
    bandwidth_cached_chart: Optional[Statistics_BandwidthCachedChart] = None
    # The BandwidthUsedChart property
    bandwidth_used_chart: Optional[Statistics_BandwidthUsedChart] = None
    # The CacheHitRate property
    cache_hit_rate: Optional[float] = None
    # The CacheHitRateChart property
    cache_hit_rate_chart: Optional[Statistics_CacheHitRateChart] = None
    # The Error3xxChart property
    error3xx_chart: Optional[Statistics_Error3xxChart] = None
    # The Error4xxChart property
    error4xx_chart: Optional[Statistics_Error4xxChart] = None
    # The Error5xxChart property
    error5xx_chart: Optional[Statistics_Error5xxChart] = None
    # The GeoTrafficDistribution property
    geo_traffic_distribution: Optional[Statistics_GeoTrafficDistribution] = None
    # The OriginResponseTimeChart property
    origin_response_time_chart: Optional[Statistics_OriginResponseTimeChart] = None
    # The OriginShieldBandwidthUsedChart property
    origin_shield_bandwidth_used_chart: Optional[Statistics_OriginShieldBandwidthUsedChart] = None
    # The OriginShieldInternalBandwidthUsedChart property
    origin_shield_internal_bandwidth_used_chart: Optional[Statistics_OriginShieldInternalBandwidthUsedChart] = None
    # The OriginTrafficChart property
    origin_traffic_chart: Optional[Statistics_OriginTrafficChart] = None
    # The PullRequestsPulledChart property
    pull_requests_pulled_chart: Optional[Statistics_PullRequestsPulledChart] = None
    # The RequestsServedChart property
    requests_served_chart: Optional[Statistics_RequestsServedChart] = None
    # The TotalBandwidthUsed property
    total_bandwidth_used: Optional[int] = None
    # The TotalOriginTraffic property
    total_origin_traffic: Optional[int] = None
    # The TotalRequestsServed property
    total_requests_served: Optional[int] = None
    # The UserBalanceHistoryChart property
    user_balance_history_chart: Optional[Statistics_UserBalanceHistoryChart] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Statistics:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Statistics
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return Statistics()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .statistics_bandwidth_cached_chart import Statistics_BandwidthCachedChart
        from .statistics_bandwidth_used_chart import Statistics_BandwidthUsedChart
        from .statistics_cache_hit_rate_chart import Statistics_CacheHitRateChart
        from .statistics_error3xx_chart import Statistics_Error3xxChart
        from .statistics_error4xx_chart import Statistics_Error4xxChart
        from .statistics_error5xx_chart import Statistics_Error5xxChart
        from .statistics_geo_traffic_distribution import Statistics_GeoTrafficDistribution
        from .statistics_origin_response_time_chart import Statistics_OriginResponseTimeChart
        from .statistics_origin_shield_bandwidth_used_chart import Statistics_OriginShieldBandwidthUsedChart
        from .statistics_origin_shield_internal_bandwidth_used_chart import Statistics_OriginShieldInternalBandwidthUsedChart
        from .statistics_origin_traffic_chart import Statistics_OriginTrafficChart
        from .statistics_pull_requests_pulled_chart import Statistics_PullRequestsPulledChart
        from .statistics_requests_served_chart import Statistics_RequestsServedChart
        from .statistics_user_balance_history_chart import Statistics_UserBalanceHistoryChart

        from .statistics_bandwidth_cached_chart import Statistics_BandwidthCachedChart
        from .statistics_bandwidth_used_chart import Statistics_BandwidthUsedChart
        from .statistics_cache_hit_rate_chart import Statistics_CacheHitRateChart
        from .statistics_error3xx_chart import Statistics_Error3xxChart
        from .statistics_error4xx_chart import Statistics_Error4xxChart
        from .statistics_error5xx_chart import Statistics_Error5xxChart
        from .statistics_geo_traffic_distribution import Statistics_GeoTrafficDistribution
        from .statistics_origin_response_time_chart import Statistics_OriginResponseTimeChart
        from .statistics_origin_shield_bandwidth_used_chart import Statistics_OriginShieldBandwidthUsedChart
        from .statistics_origin_shield_internal_bandwidth_used_chart import Statistics_OriginShieldInternalBandwidthUsedChart
        from .statistics_origin_traffic_chart import Statistics_OriginTrafficChart
        from .statistics_pull_requests_pulled_chart import Statistics_PullRequestsPulledChart
        from .statistics_requests_served_chart import Statistics_RequestsServedChart
        from .statistics_user_balance_history_chart import Statistics_UserBalanceHistoryChart

        fields: Dict[str, Callable[[Any], None]] = {
            "AverageOriginResponseTime": lambda n : setattr(self, 'average_origin_response_time', n.get_int_value()),
            "BandwidthCachedChart": lambda n : setattr(self, 'bandwidth_cached_chart', n.get_object_value(Statistics_BandwidthCachedChart)),
            "BandwidthUsedChart": lambda n : setattr(self, 'bandwidth_used_chart', n.get_object_value(Statistics_BandwidthUsedChart)),
            "CacheHitRate": lambda n : setattr(self, 'cache_hit_rate', n.get_float_value()),
            "CacheHitRateChart": lambda n : setattr(self, 'cache_hit_rate_chart', n.get_object_value(Statistics_CacheHitRateChart)),
            "Error3xxChart": lambda n : setattr(self, 'error3xx_chart', n.get_object_value(Statistics_Error3xxChart)),
            "Error4xxChart": lambda n : setattr(self, 'error4xx_chart', n.get_object_value(Statistics_Error4xxChart)),
            "Error5xxChart": lambda n : setattr(self, 'error5xx_chart', n.get_object_value(Statistics_Error5xxChart)),
            "GeoTrafficDistribution": lambda n : setattr(self, 'geo_traffic_distribution', n.get_object_value(Statistics_GeoTrafficDistribution)),
            "OriginResponseTimeChart": lambda n : setattr(self, 'origin_response_time_chart', n.get_object_value(Statistics_OriginResponseTimeChart)),
            "OriginShieldBandwidthUsedChart": lambda n : setattr(self, 'origin_shield_bandwidth_used_chart', n.get_object_value(Statistics_OriginShieldBandwidthUsedChart)),
            "OriginShieldInternalBandwidthUsedChart": lambda n : setattr(self, 'origin_shield_internal_bandwidth_used_chart', n.get_object_value(Statistics_OriginShieldInternalBandwidthUsedChart)),
            "OriginTrafficChart": lambda n : setattr(self, 'origin_traffic_chart', n.get_object_value(Statistics_OriginTrafficChart)),
            "PullRequestsPulledChart": lambda n : setattr(self, 'pull_requests_pulled_chart', n.get_object_value(Statistics_PullRequestsPulledChart)),
            "RequestsServedChart": lambda n : setattr(self, 'requests_served_chart', n.get_object_value(Statistics_RequestsServedChart)),
            "TotalBandwidthUsed": lambda n : setattr(self, 'total_bandwidth_used', n.get_int_value()),
            "TotalOriginTraffic": lambda n : setattr(self, 'total_origin_traffic', n.get_int_value()),
            "TotalRequestsServed": lambda n : setattr(self, 'total_requests_served', n.get_int_value()),
            "UserBalanceHistoryChart": lambda n : setattr(self, 'user_balance_history_chart', n.get_object_value(Statistics_UserBalanceHistoryChart)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("AverageOriginResponseTime", self.average_origin_response_time)
        writer.write_object_value("BandwidthCachedChart", self.bandwidth_cached_chart)
        writer.write_object_value("BandwidthUsedChart", self.bandwidth_used_chart)
        writer.write_float_value("CacheHitRate", self.cache_hit_rate)
        writer.write_object_value("CacheHitRateChart", self.cache_hit_rate_chart)
        writer.write_object_value("Error3xxChart", self.error3xx_chart)
        writer.write_object_value("Error4xxChart", self.error4xx_chart)
        writer.write_object_value("Error5xxChart", self.error5xx_chart)
        writer.write_object_value("GeoTrafficDistribution", self.geo_traffic_distribution)
        writer.write_object_value("OriginResponseTimeChart", self.origin_response_time_chart)
        writer.write_object_value("OriginShieldBandwidthUsedChart", self.origin_shield_bandwidth_used_chart)
        writer.write_object_value("OriginShieldInternalBandwidthUsedChart", self.origin_shield_internal_bandwidth_used_chart)
        writer.write_object_value("OriginTrafficChart", self.origin_traffic_chart)
        writer.write_object_value("PullRequestsPulledChart", self.pull_requests_pulled_chart)
        writer.write_object_value("RequestsServedChart", self.requests_served_chart)
        writer.write_int_value("TotalBandwidthUsed", self.total_bandwidth_used)
        writer.write_int_value("TotalOriginTraffic", self.total_origin_traffic)
        writer.write_int_value("TotalRequestsServed", self.total_requests_served)
        writer.write_object_value("UserBalanceHistoryChart", self.user_balance_history_chart)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ...models.pull_zone.pull_zone import PullZone
    from ...models.pull_zone.pull_zone_create import PullZoneCreate
    from .add_allowed_referrer.add_allowed_referrer_request_builder import AddAllowedReferrerRequestBuilder
    from .add_blocked_ip.add_blocked_ip_request_builder import AddBlockedIpRequestBuilder
    from .add_blocked_referrer.add_blocked_referrer_request_builder import AddBlockedReferrerRequestBuilder
    from .add_certificate.add_certificate_request_builder import AddCertificateRequestBuilder
    from .add_hostname.add_hostname_request_builder import AddHostnameRequestBuilder
    from .edgerules.edgerules_request_builder import EdgerulesRequestBuilder
    from .optimizer.optimizer_request_builder import OptimizerRequestBuilder
    from .originshield.originshield_request_builder import OriginshieldRequestBuilder
    from .purge_cache.purge_cache_request_builder import PurgeCacheRequestBuilder
    from .remove_allowed_referrer.remove_allowed_referrer_request_builder import RemoveAllowedReferrerRequestBuilder
    from .remove_blocked_ip.remove_blocked_ip_request_builder import RemoveBlockedIpRequestBuilder
    from .remove_blocked_referrer.remove_blocked_referrer_request_builder import RemoveBlockedReferrerRequestBuilder
    from .remove_certificate.remove_certificate_request_builder import RemoveCertificateRequestBuilder
    from .remove_hostname.remove_hostname_request_builder import RemoveHostnameRequestBuilder
    from .reset_security_key.reset_security_key_request_builder import ResetSecurityKeyRequestBuilder
    from .safehop.safehop_request_builder import SafehopRequestBuilder
    from .set_force_s_s_l.set_force_s_s_l_request_builder import SetForceSSLRequestBuilder
    from .waf.waf_request_builder import WafRequestBuilder

class ItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /pullzone/{-id}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new ItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/pullzone/{%2Did}?includeCertificate={includeCertificate}", path_parameters)
    
    async def delete(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> None:
        """
        [DeletePullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_delete)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: None
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        return await self.request_adapter.send_no_response_content_async(request_info, None)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ItemRequestBuilderGetQueryParameters]] = None) -> Optional[PullZone]:
        """
        [GetPullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_index2)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PullZone]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.pull_zone.pull_zone import PullZone

        return await self.request_adapter.send_async(request_info, PullZone, None)
    
    async def post(self,body: PullZoneCreate, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[PullZone]:
        """
        [UpdatePullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_updatepullzone)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PullZone]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.pull_zone.pull_zone import PullZone

        return await self.request_adapter.send_async(request_info, PullZone, None)
    
    def to_delete_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [DeletePullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_delete)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.DELETE, '{+baseurl}/pullzone/{%2Did}', self.path_parameters)
        request_info.configure(request_configuration)
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ItemRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        [GetPullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_index2)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: PullZoneCreate, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [UpdatePullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_updatepullzone)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, '{+baseurl}/pullzone/{%2Did}', self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> ItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def add_allowed_referrer(self) -> AddAllowedReferrerRequestBuilder:
        """
        The addAllowedReferrer property
        """
        from .add_allowed_referrer.add_allowed_referrer_request_builder import AddAllowedReferrerRequestBuilder

        return AddAllowedReferrerRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def add_blocked_ip(self) -> AddBlockedIpRequestBuilder:
        """
        The addBlockedIp property
        """
        from .add_blocked_ip.add_blocked_ip_request_builder import AddBlockedIpRequestBuilder

        return AddBlockedIpRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def add_blocked_referrer(self) -> AddBlockedReferrerRequestBuilder:
        """
        The addBlockedReferrer property
        """
        from .add_blocked_referrer.add_blocked_referrer_request_builder import AddBlockedReferrerRequestBuilder

        return AddBlockedReferrerRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def add_certificate(self) -> AddCertificateRequestBuilder:
        """
        The addCertificate property
        """
        from .add_certificate.add_certificate_request_builder import AddCertificateRequestBuilder

        return AddCertificateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def add_hostname(self) -> AddHostnameRequestBuilder:
        """
        The addHostname property
        """
        from .add_hostname.add_hostname_request_builder import AddHostnameRequestBuilder

        return AddHostnameRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def edgerules(self) -> EdgerulesRequestBuilder:
        """
        The edgerules property
        """
        from .edgerules.edgerules_request_builder import EdgerulesRequestBuilder

        return EdgerulesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def optimizer(self) -> OptimizerRequestBuilder:
        """
        The optimizer property
        """
        from .optimizer.optimizer_request_builder import OptimizerRequestBuilder

        return OptimizerRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def originshield(self) -> OriginshieldRequestBuilder:
        """
        The originshield property
        """
        from .originshield.originshield_request_builder import OriginshieldRequestBuilder

        return OriginshieldRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def purge_cache(self) -> PurgeCacheRequestBuilder:
        """
        The purgeCache property
        """
        from .purge_cache.purge_cache_request_builder import PurgeCacheRequestBuilder

        return PurgeCacheRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def remove_allowed_referrer(self) -> RemoveAllowedReferrerRequestBuilder:
        """
        The removeAllowedReferrer property
        """
        from .remove_allowed_referrer.remove_allowed_referrer_request_builder import RemoveAllowedReferrerRequestBuilder

        return RemoveAllowedReferrerRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def remove_blocked_ip(self) -> RemoveBlockedIpRequestBuilder:
        """
        The removeBlockedIp property
        """
        from .remove_blocked_ip.remove_blocked_ip_request_builder import RemoveBlockedIpRequestBuilder

        return RemoveBlockedIpRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def remove_blocked_referrer(self) -> RemoveBlockedReferrerRequestBuilder:
        """
        The removeBlockedReferrer property
        """
        from .remove_blocked_referrer.remove_blocked_referrer_request_builder import RemoveBlockedReferrerRequestBuilder

        return RemoveBlockedReferrerRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def remove_certificate(self) -> RemoveCertificateRequestBuilder:
        """
        The removeCertificate property
        """
        from .remove_certificate.remove_certificate_request_builder import RemoveCertificateRequestBuilder

        return RemoveCertificateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def remove_hostname(self) -> RemoveHostnameRequestBuilder:
        """
        The removeHostname property
        """
        from .remove_hostname.remove_hostname_request_builder import RemoveHostnameRequestBuilder

        return RemoveHostnameRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def reset_security_key(self) -> ResetSecurityKeyRequestBuilder:
        """
        The resetSecurityKey property
        """
        from .reset_security_key.reset_security_key_request_builder import ResetSecurityKeyRequestBuilder

        return ResetSecurityKeyRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def safehop(self) -> SafehopRequestBuilder:
        """
        The safehop property
        """
        from .safehop.safehop_request_builder import SafehopRequestBuilder

        return SafehopRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def set_force_s_s_l(self) -> SetForceSSLRequestBuilder:
        """
        The setForceSSL property
        """
        from .set_force_s_s_l.set_force_s_s_l_request_builder import SetForceSSLRequestBuilder

        return SetForceSSLRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def waf(self) -> WafRequestBuilder:
        """
        The waf property
        """
        from .waf.waf_request_builder import WafRequestBuilder

        return WafRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class ItemRequestBuilderDeleteRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class ItemRequestBuilderGetQueryParameters():
        """
        [GetPullZone API Docs](https://docs.bunny.net/reference/pullzonepublic_index2)
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "include_certificate":
                return "includeCertificate"
            return original_name
        
        # Determines if the result hostnames should contain the SSL certificate
        include_certificate: Optional[bool] = None

    
    @dataclass
    class ItemRequestBuilderGetRequestConfiguration(RequestConfiguration[ItemRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class ItemRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .statistics_get_response_average_compression_chart import StatisticsGetResponse_AverageCompressionChart
    from .statistics_get_response_average_processing_time_chart import StatisticsGetResponse_AverageProcessingTimeChart
    from .statistics_get_response_requests_optimized_chart import StatisticsGetResponse_RequestsOptimizedChart
    from .statistics_get_response_traffic_saved_chart import StatisticsGetResponse_TrafficSavedChart

@dataclass
class StatisticsGetResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # Average compression chart of the responses
    average_compression_chart: Optional[StatisticsGetResponse_AverageCompressionChart] = None
    # The average compression ratio of CDN responses
    average_compression_ratio: Optional[float] = None
    # The average processing time of each request
    average_processing_time: Optional[float] = None
    # The AverageProcessingTimeChart property
    average_processing_time_chart: Optional[StatisticsGetResponse_AverageProcessingTimeChart] = None
    # The constructed chart of optimized requests
    requests_optimized_chart: Optional[StatisticsGetResponse_RequestsOptimizedChart] = None
    # The total number of optimized requests
    total_requests_optimized: Optional[float] = None
    # The total requests saved
    total_traffic_saved: Optional[float] = None
    # The constructed chart of saved traffic
    traffic_saved_chart: Optional[StatisticsGetResponse_TrafficSavedChart] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatisticsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatisticsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatisticsGetResponse()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .statistics_get_response_average_compression_chart import StatisticsGetResponse_AverageCompressionChart
        from .statistics_get_response_average_processing_time_chart import StatisticsGetResponse_AverageProcessingTimeChart
        from .statistics_get_response_requests_optimized_chart import StatisticsGetResponse_RequestsOptimizedChart
        from .statistics_get_response_traffic_saved_chart import StatisticsGetResponse_TrafficSavedChart

        from .statistics_get_response_average_compression_chart import StatisticsGetResponse_AverageCompressionChart
        from .statistics_get_response_average_processing_time_chart import StatisticsGetResponse_AverageProcessingTimeChart
        from .statistics_get_response_requests_optimized_chart import StatisticsGetResponse_RequestsOptimizedChart
        from .statistics_get_response_traffic_saved_chart import StatisticsGetResponse_TrafficSavedChart

        fields: Dict[str, Callable[[Any], None]] = {
            "AverageCompressionChart": lambda n : setattr(self, 'average_compression_chart', n.get_object_value(StatisticsGetResponse_AverageCompressionChart)),
            "AverageCompressionRatio": lambda n : setattr(self, 'average_compression_ratio', n.get_float_value()),
            "AverageProcessingTime": lambda n : setattr(self, 'average_processing_time', n.get_float_value()),
            "AverageProcessingTimeChart": lambda n : setattr(self, 'average_processing_time_chart', n.get_object_value(StatisticsGetResponse_AverageProcessingTimeChart)),
            "RequestsOptimizedChart": lambda n : setattr(self, 'requests_optimized_chart', n.get_object_value(StatisticsGetResponse_RequestsOptimizedChart)),
            "TotalRequestsOptimized": lambda n : setattr(self, 'total_requests_optimized', n.get_float_value()),
            "TotalTrafficSaved": lambda n : setattr(self, 'total_traffic_saved', n.get_float_value()),
            "TrafficSavedChart": lambda n : setattr(self, 'traffic_saved_chart', n.get_object_value(StatisticsGetResponse_TrafficSavedChart)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("AverageCompressionChart", self.average_compression_chart)
        writer.write_float_value("AverageCompressionRatio", self.average_compression_ratio)
        writer.write_float_value("AverageProcessingTime", self.average_processing_time)
        writer.write_object_value("AverageProcessingTimeChart", self.average_processing_time_chart)
        writer.write_object_value("RequestsOptimizedChart", self.requests_optimized_chart)
        writer.write_float_value("TotalRequestsOptimized", self.total_requests_optimized)
        writer.write_float_value("TotalTrafficSaved", self.total_traffic_saved)
        writer.write_object_value("TrafficSavedChart", self.traffic_saved_chart)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class ScriptRelease(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The Author property
    author: Optional[str] = None
    # The AuthorEmail property
    author_email: Optional[str] = None
    # The Code property
    code: Optional[str] = None
    # The DatePublished property
    date_published: Optional[datetime.datetime] = None
    # The DateReleased property
    date_released: Optional[datetime.datetime] = None
    # The Deleted property
    deleted: Optional[bool] = None
    # The Id property
    id: Optional[int] = None
    # The Note property
    note: Optional[str] = None
    # The Status property
    status: Optional[float] = None
    # The Uuid property
    uuid: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ScriptRelease:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ScriptRelease
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ScriptRelease()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "Author": lambda n : setattr(self, 'author', n.get_str_value()),
            "AuthorEmail": lambda n : setattr(self, 'author_email', n.get_str_value()),
            "Code": lambda n : setattr(self, 'code', n.get_str_value()),
            "DatePublished": lambda n : setattr(self, 'date_published', n.get_datetime_value()),
            "DateReleased": lambda n : setattr(self, 'date_released', n.get_datetime_value()),
            "Deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "Id": lambda n : setattr(self, 'id', n.get_int_value()),
            "Note": lambda n : setattr(self, 'note', n.get_str_value()),
            "Status": lambda n : setattr(self, 'status', n.get_float_value()),
            "Uuid": lambda n : setattr(self, 'uuid', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("Author", self.author)
        writer.write_str_value("AuthorEmail", self.author_email)
        writer.write_str_value("Code", self.code)
        writer.write_datetime_value("DatePublished", self.date_published)
        writer.write_datetime_value("DateReleased", self.date_released)
        writer.write_bool_value("Deleted", self.deleted)
        writer.write_int_value("Id", self.id)
        writer.write_str_value("Note", self.note)
        writer.write_float_value("Status", self.status)
        writer.write_str_value("Uuid", self.uuid)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .remove_allowed_referrer_post_request_body import RemoveAllowedReferrerPostRequestBody

class RemoveAllowedReferrerRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /videolibrary/{id}/removeAllowedReferrer
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new RemoveAllowedReferrerRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/videolibrary/{id}/removeAllowedReferrer", path_parameters)
    
    async def post(self,body: RemoveAllowedReferrerPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> None:
        """
        [RemoveAllowedReferer API Docs](https://docs.bunny.net/reference/videolibrarypublic_removeallowedreferrer)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: None
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        return await self.request_adapter.send_no_response_content_async(request_info, None)
    
    def to_post_request_information(self,body: RemoveAllowedReferrerPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [RemoveAllowedReferer API Docs](https://docs.bunny.net/reference/videolibrarypublic_removeallowedreferrer)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> RemoveAllowedReferrerRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: RemoveAllowedReferrerRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return RemoveAllowedReferrerRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class RemoveAllowedReferrerRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .statistics_get_response import StatisticsGetResponse

class StatisticsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /dnszone/{-id}/statistics
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new StatisticsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/dnszone/{%2Did}/statistics{?dateFrom,dateTo}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[StatisticsRequestBuilderGetQueryParameters]] = None) -> Optional[StatisticsGetResponse]:
        """
        [GetDnsQueryStatistics API Docs](https://docs.bunny.net/reference/dnszonepublic_statistics)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[StatisticsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .statistics_get_response import StatisticsGetResponse

        return await self.request_adapter.send_async(request_info, StatisticsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[StatisticsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        [GetDnsQueryStatistics API Docs](https://docs.bunny.net/reference/dnszonepublic_statistics)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> StatisticsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: StatisticsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return StatisticsRequestBuilder(self.request_adapter, raw_url)
    
    import datetime

    @dataclass
    class StatisticsRequestBuilderGetQueryParameters():
        import datetime

        """
        [GetDnsQueryStatistics API Docs](https://docs.bunny.net/reference/dnszonepublic_statistics)
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "date_from":
                return "dateFrom"
            if original_name == "date_to":
                return "dateTo"
            return original_name
        
        # The start date of the statistics. If no value is passed, the last 30 days will be returned
        date_from: Optional[datetime.datetime] = None

        # The end date of the statistics. If no value is passed, the last 30 days will be returned
        date_to: Optional[datetime.datetime] = None

    
    @dataclass
    class StatisticsRequestBuilderGetRequestConfiguration(RequestConfiguration[StatisticsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class PostRequestBody(AdditionalDataHolder, Parsable):
    """
    The template for adding optional properties.
    """
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The CustomNameserversEnabled property
    custom_nameservers_enabled: Optional[bool] = None
    # The LogAnonymizationType property
    log_anonymization_type: Optional[float] = None
    # The LoggingEnabled property
    logging_enabled: Optional[bool] = None
    # Determines if the log anonymization should be enabled
    logging_i_p_anonymization_enabled: Optional[bool] = None
    # The Nameserver1 property
    nameserver1: Optional[str] = None
    # The Nameserver2 property
    nameserver2: Optional[str] = None
    # The SoaEmail property
    soa_email: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PostRequestBody()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "CustomNameserversEnabled": lambda n : setattr(self, 'custom_nameservers_enabled', n.get_bool_value()),
            "LogAnonymizationType": lambda n : setattr(self, 'log_anonymization_type', n.get_float_value()),
            "LoggingEnabled": lambda n : setattr(self, 'logging_enabled', n.get_bool_value()),
            "LoggingIPAnonymizationEnabled": lambda n : setattr(self, 'logging_i_p_anonymization_enabled', n.get_bool_value()),
            "Nameserver1": lambda n : setattr(self, 'nameserver1', n.get_str_value()),
            "Nameserver2": lambda n : setattr(self, 'nameserver2', n.get_str_value()),
            "SoaEmail": lambda n : setattr(self, 'soa_email', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("CustomNameserversEnabled", self.custom_nameservers_enabled)
        writer.write_float_value("LogAnonymizationType", self.log_anonymization_type)
        writer.write_bool_value("LoggingEnabled", self.logging_enabled)
        writer.write_bool_value("LoggingIPAnonymizationEnabled", self.logging_i_p_anonymization_enabled)
        writer.write_str_value("Nameserver1", self.nameserver1)
        writer.write_str_value("Nameserver2", self.nameserver2)
        writer.write_str_value("SoaEmail", self.soa_email)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .statistics_get_response_requests_retried_chart import StatisticsGetResponse_RequestsRetriedChart
    from .statistics_get_response_requests_saved_chart import StatisticsGetResponse_RequestsSavedChart

@dataclass
class StatisticsGetResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The constructed chart of requests retried
    requests_retried_chart: Optional[StatisticsGetResponse_RequestsRetriedChart] = None
    # The constructed chart of requests saved
    requests_saved_chart: Optional[StatisticsGetResponse_RequestsSavedChart] = None
    # The total number of retried requests
    total_requests_retried: Optional[int] = None
    # The total number of saved requests
    total_requests_saved: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatisticsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatisticsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatisticsGetResponse()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .statistics_get_response_requests_retried_chart import StatisticsGetResponse_RequestsRetriedChart
        from .statistics_get_response_requests_saved_chart import StatisticsGetResponse_RequestsSavedChart

        from .statistics_get_response_requests_retried_chart import StatisticsGetResponse_RequestsRetriedChart
        from .statistics_get_response_requests_saved_chart import StatisticsGetResponse_RequestsSavedChart

        fields: Dict[str, Callable[[Any], None]] = {
            "RequestsRetriedChart": lambda n : setattr(self, 'requests_retried_chart', n.get_object_value(StatisticsGetResponse_RequestsRetriedChart)),
            "RequestsSavedChart": lambda n : setattr(self, 'requests_saved_chart', n.get_object_value(StatisticsGetResponse_RequestsSavedChart)),
            "TotalRequestsRetried": lambda n : setattr(self, 'total_requests_retried', n.get_int_value()),
            "TotalRequestsSaved": lambda n : setattr(self, 'total_requests_saved', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("RequestsRetriedChart", self.requests_retried_chart)
        writer.write_object_value("RequestsSavedChart", self.requests_saved_chart)
        writer.write_int_value("TotalRequestsRetried", self.total_requests_retried)
        writer.write_int_value("TotalRequestsSaved", self.total_requests_saved)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .statistics_get_response_country_view_counts import StatisticsGetResponse_countryViewCounts
    from .statistics_get_response_country_watch_time import StatisticsGetResponse_countryWatchTime
    from .statistics_get_response_views_chart import StatisticsGetResponse_viewsChart
    from .statistics_get_response_watch_time_chart import StatisticsGetResponse_watchTimeChart

@dataclass
class StatisticsGetResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The countryViewCounts property
    country_view_counts: Optional[StatisticsGetResponse_countryViewCounts] = None
    # The countryWatchTime property
    country_watch_time: Optional[StatisticsGetResponse_countryWatchTime] = None
    # The engagementScore property
    engagement_score: Optional[int] = None
    # The viewsChart property
    views_chart: Optional[StatisticsGetResponse_viewsChart] = None
    # The watchTimeChart property
    watch_time_chart: Optional[StatisticsGetResponse_watchTimeChart] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatisticsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatisticsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatisticsGetResponse()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .statistics_get_response_country_view_counts import StatisticsGetResponse_countryViewCounts
        from .statistics_get_response_country_watch_time import StatisticsGetResponse_countryWatchTime
        from .statistics_get_response_views_chart import StatisticsGetResponse_viewsChart
        from .statistics_get_response_watch_time_chart import StatisticsGetResponse_watchTimeChart

        from .statistics_get_response_country_view_counts import StatisticsGetResponse_countryViewCounts
        from .statistics_get_response_country_watch_time import StatisticsGetResponse_countryWatchTime
        from .statistics_get_response_views_chart import StatisticsGetResponse_viewsChart
        from .statistics_get_response_watch_time_chart import StatisticsGetResponse_watchTimeChart

        fields: Dict[str, Callable[[Any], None]] = {
            "countryViewCounts": lambda n : setattr(self, 'country_view_counts', n.get_object_value(StatisticsGetResponse_countryViewCounts)),
            "countryWatchTime": lambda n : setattr(self, 'country_watch_time', n.get_object_value(StatisticsGetResponse_countryWatchTime)),
            "engagementScore": lambda n : setattr(self, 'engagement_score', n.get_int_value()),
            "viewsChart": lambda n : setattr(self, 'views_chart', n.get_object_value(StatisticsGetResponse_viewsChart)),
            "watchTimeChart": lambda n : setattr(self, 'watch_time_chart', n.get_object_value(StatisticsGetResponse_watchTimeChart)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("countryViewCounts", self.country_view_counts)
        writer.write_object_value("countryWatchTime", self.country_watch_time)
        writer.write_int_value("engagementScore", self.engagement_score)
        writer.write_object_value("viewsChart", self.views_chart)
        writer.write_object_value("watchTimeChart", self.watch_time_chart)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .url import Url

@dataclass
class AbuseCase(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The ActualUrl property
    actual_url: Optional[str] = None
    # The DateCreated property
    date_created: Optional[datetime.datetime] = None
    # The DateUpdated property
    date_updated: Optional[datetime.datetime] = None
    # The Deadline property
    deadline: Optional[datetime.datetime] = None
    # The Id property
    id: Optional[int] = None
    # The Message property
    message: Optional[str] = None
    # The Path property
    path: Optional[str] = None
    # The PullZoneId property
    pull_zone_id: Optional[int] = None
    # The PullZoneName property
    pull_zone_name: Optional[str] = None
    # The Status property
    status: Optional[float] = None
    # The Urls property
    urls: Optional[List[Url]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AbuseCase:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AbuseCase
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AbuseCase()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .url import Url

        from .url import Url

        fields: Dict[str, Callable[[Any], None]] = {
            "ActualUrl": lambda n : setattr(self, 'actual_url', n.get_str_value()),
            "DateCreated": lambda n : setattr(self, 'date_created', n.get_datetime_value()),
            "DateUpdated": lambda n : setattr(self, 'date_updated', n.get_datetime_value()),
            "Deadline": lambda n : setattr(self, 'deadline', n.get_datetime_value()),
            "Id": lambda n : setattr(self, 'id', n.get_int_value()),
            "Message": lambda n : setattr(self, 'message', n.get_str_value()),
            "Path": lambda n : setattr(self, 'path', n.get_str_value()),
            "PullZoneId": lambda n : setattr(self, 'pull_zone_id', n.get_int_value()),
            "PullZoneName": lambda n : setattr(self, 'pull_zone_name', n.get_str_value()),
            "Status": lambda n : setattr(self, 'status', n.get_float_value()),
            "Urls": lambda n : setattr(self, 'urls', n.get_collection_of_object_values(Url)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("ActualUrl", self.actual_url)
        writer.write_datetime_value("DateCreated", self.date_created)
        writer.write_datetime_value("DateUpdated", self.date_updated)
        writer.write_datetime_value("Deadline", self.deadline)
        writer.write_int_value("Id", self.id)
        writer.write_str_value("Message", self.message)
        writer.write_str_value("Path", self.path)
        writer.write_int_value("PullZoneId", self.pull_zone_id)
        writer.write_str_value("PullZoneName", self.pull_zone_name)
        writer.write_float_value("Status", self.status)
        writer.write_collection_of_object_values("Urls", self.urls)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class File(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The ArrayNumber property
    array_number: Optional[int] = None
    # The Checksum property
    checksum: Optional[str] = None
    # The ContentType property
    content_type: Optional[str] = None
    # The DateCreated property
    date_created: Optional[datetime.datetime] = None
    # The Guid property
    guid: Optional[str] = None
    # The IsDirectory property
    is_directory: Optional[bool] = None
    # The LastChanged property
    last_changed: Optional[datetime.datetime] = None
    # The Length property
    length: Optional[int] = None
    # The ObjectName property
    object_name: Optional[str] = None
    # The Path property
    path: Optional[str] = None
    # The ReplicatedZones property
    replicated_zones: Optional[str] = None
    # The ServerId property
    server_id: Optional[int] = None
    # The StorageZoneId property
    storage_zone_id: Optional[int] = None
    # The StorageZoneName property
    storage_zone_name: Optional[str] = None
    # The UserId property
    user_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> File:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: File
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return File()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "ArrayNumber": lambda n : setattr(self, 'array_number', n.get_int_value()),
            "Checksum": lambda n : setattr(self, 'checksum', n.get_str_value()),
            "ContentType": lambda n : setattr(self, 'content_type', n.get_str_value()),
            "DateCreated": lambda n : setattr(self, 'date_created', n.get_datetime_value()),
            "Guid": lambda n : setattr(self, 'guid', n.get_str_value()),
            "IsDirectory": lambda n : setattr(self, 'is_directory', n.get_bool_value()),
            "LastChanged": lambda n : setattr(self, 'last_changed', n.get_datetime_value()),
            "Length": lambda n : setattr(self, 'length', n.get_int_value()),
            "ObjectName": lambda n : setattr(self, 'object_name', n.get_str_value()),
            "Path": lambda n : setattr(self, 'path', n.get_str_value()),
            "ReplicatedZones": lambda n : setattr(self, 'replicated_zones', n.get_str_value()),
            "ServerId": lambda n : setattr(self, 'server_id', n.get_int_value()),
            "StorageZoneId": lambda n : setattr(self, 'storage_zone_id', n.get_int_value()),
            "StorageZoneName": lambda n : setattr(self, 'storage_zone_name', n.get_str_value()),
            "UserId": lambda n : setattr(self, 'user_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("ArrayNumber", self.array_number)
        writer.write_str_value("Checksum", self.checksum)
        writer.write_str_value("ContentType", self.content_type)
        writer.write_datetime_value("DateCreated", self.date_created)
        writer.write_str_value("Guid", self.guid)
        writer.write_bool_value("IsDirectory", self.is_directory)
        writer.write_datetime_value("LastChanged", self.last_changed)
        writer.write_int_value("Length", self.length)
        writer.write_str_value("ObjectName", self.object_name)
        writer.write_str_value("Path", self.path)
        writer.write_str_value("ReplicatedZones", self.replicated_zones)
        writer.write_int_value("ServerId", self.server_id)
        writer.write_int_value("StorageZoneId", self.storage_zone_id)
        writer.write_str_value("StorageZoneName", self.storage_zone_name)
        writer.write_str_value("UserId", self.user_id)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ....models.manage_collections.collection import Collection
    from .collections_get_response import CollectionsGetResponse
    from .get_order_by_query_parameter_type import GetOrderByQueryParameterType
    from .item.with_collection_item_request_builder import WithCollectionItemRequestBuilder

class CollectionsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /library/{libraryId}/collections
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new CollectionsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/library/{libraryId}/collections?includeThumbnails={includeThumbnails}&itemsPerPage={itemsPerPage}&orderBy={orderBy}&page={page}{&search}", path_parameters)
    
    def by_collection_id(self,collection_id: str) -> WithCollectionItemRequestBuilder:
        """
        Gets an item from the StreamApiClient.library.item.collections.item collection
        param collection_id: Unique identifier of the item
        Returns: WithCollectionItemRequestBuilder
        """
        if collection_id is None:
            raise TypeError("collection_id cannot be null.")
        from .item.with_collection_item_request_builder import WithCollectionItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["collectionId"] = collection_id
        return WithCollectionItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[CollectionsRequestBuilderGetQueryParameters]] = None) -> Optional[CollectionsGetResponse]:
        """
        [GetCollectionList API Docs](https://docs.bunny.net/reference/collection_list)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CollectionsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .collections_get_response import CollectionsGetResponse

        return await self.request_adapter.send_async(request_info, CollectionsGetResponse, None)
    
    async def post(self,body: Collection, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[Collection]:
        """
        [CreateCollection API Docs](https://docs.bunny.net/reference/collection_createcollection)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[Collection]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ....models.manage_collections.collection import Collection

        return await self.request_adapter.send_async(request_info, Collection, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[CollectionsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        [GetCollectionList API Docs](https://docs.bunny.net/reference/collection_list)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: Collection, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [CreateCollection API Docs](https://docs.bunny.net/reference/collection_createcollection)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, '{+baseurl}/library/{libraryId}/collections', self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> CollectionsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CollectionsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CollectionsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class CollectionsRequestBuilderGetQueryParameters():
        """
        [GetCollectionList API Docs](https://docs.bunny.net/reference/collection_list)
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "include_thumbnails":
                return "includeThumbnails"
            if original_name == "items_per_page":
                return "itemsPerPage"
            if original_name == "order_by":
                return "orderBy"
            if original_name == "page":
                return "page"
            if original_name == "search":
                return "search"
            return original_name
        
        include_thumbnails: Optional[bool] = None

        items_per_page: Optional[int] = None

        order_by: Optional[GetOrderByQueryParameterType] = None

        page: Optional[int] = None

        search: Optional[str] = None

    
    @dataclass
    class CollectionsRequestBuilderGetRequestConfiguration(RequestConfiguration[CollectionsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class CollectionsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .properties_auto_optimize import Properties_auto_optimize
    from .properties_crop_gravity import Properties_crop_gravity
    from .properties_optimizer import Properties_optimizer

@dataclass
class Properties(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The aspect_ratio property
    aspect_ratio: Optional[str] = None
    # The auto_optimize property
    auto_optimize: Optional[Properties_auto_optimize] = None
    # The blur property
    blur: Optional[str] = None
    # The brightness property
    brightness: Optional[str] = None
    # The contrast property
    contrast: Optional[str] = None
    # The crop property
    crop: Optional[str] = None
    # The crop_gravity property
    crop_gravity: Optional[Properties_crop_gravity] = None
    # The flip property
    flip: Optional[str] = None
    # The flop property
    flop: Optional[str] = None
    # The gamma property
    gamma: Optional[str] = None
    # The height property
    height: Optional[str] = None
    # The hue property
    hue: Optional[str] = None
    # The optimizer property
    optimizer: Optional[Properties_optimizer] = None
    # The quality property
    quality: Optional[str] = None
    # The saturation property
    saturation: Optional[str] = None
    # The sharpen property
    sharpen: Optional[str] = None
    # The width property
    width: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Properties:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Properties
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return Properties()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .properties_auto_optimize import Properties_auto_optimize
        from .properties_crop_gravity import Properties_crop_gravity
        from .properties_optimizer import Properties_optimizer

        from .properties_auto_optimize import Properties_auto_optimize
        from .properties_crop_gravity import Properties_crop_gravity
        from .properties_optimizer import Properties_optimizer

        fields: Dict[str, Callable[[Any], None]] = {
            "aspect_ratio": lambda n : setattr(self, 'aspect_ratio', n.get_str_value()),
            "auto_optimize": lambda n : setattr(self, 'auto_optimize', n.get_enum_value(Properties_auto_optimize)),
            "blur": lambda n : setattr(self, 'blur', n.get_str_value()),
            "brightness": lambda n : setattr(self, 'brightness', n.get_str_value()),
            "contrast": lambda n : setattr(self, 'contrast', n.get_str_value()),
            "crop": lambda n : setattr(self, 'crop', n.get_str_value()),
            "crop_gravity": lambda n : setattr(self, 'crop_gravity', n.get_enum_value(Properties_crop_gravity)),
            "flip": lambda n : setattr(self, 'flip', n.get_str_value()),
            "flop": lambda n : setattr(self, 'flop', n.get_str_value()),
            "gamma": lambda n : setattr(self, 'gamma', n.get_str_value()),
            "height": lambda n : setattr(self, 'height', n.get_str_value()),
            "hue": lambda n : setattr(self, 'hue', n.get_str_value()),
            "optimizer": lambda n : setattr(self, 'optimizer', n.get_enum_value(Properties_optimizer)),
            "quality": lambda n : setattr(self, 'quality', n.get_str_value()),
            "saturation": lambda n : setattr(self, 'saturation', n.get_str_value()),
            "sharpen": lambda n : setattr(self, 'sharpen', n.get_str_value()),
            "width": lambda n : setattr(self, 'width', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("aspect_ratio", self.aspect_ratio)
        writer.write_enum_value("auto_optimize", self.auto_optimize)
        writer.write_str_value("blur", self.blur)
        writer.write_str_value("brightness", self.brightness)
        writer.write_str_value("contrast", self.contrast)
        writer.write_str_value("crop", self.crop)
        writer.write_enum_value("crop_gravity", self.crop_gravity)
        writer.write_str_value("flip", self.flip)
        writer.write_str_value("flop", self.flop)
        writer.write_str_value("gamma", self.gamma)
        writer.write_str_value("height", self.height)
        writer.write_str_value("hue", self.hue)
        writer.write_enum_value("optimizer", self.optimizer)
        writer.write_str_value("quality", self.quality)
        writer.write_str_value("saturation", self.saturation)
        writer.write_str_value("sharpen", self.sharpen)
        writer.write_str_value("width", self.width)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .collections.collections_request_builder import CollectionsRequestBuilder
    from .statistics.statistics_request_builder import StatisticsRequestBuilder
    from .videos.videos_request_builder import VideosRequestBuilder

class WithLibraryItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /library/{libraryId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new WithLibraryItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/library/{libraryId}", path_parameters)
    
    @property
    def collections(self) -> CollectionsRequestBuilder:
        """
        The collections property
        """
        from .collections.collections_request_builder import CollectionsRequestBuilder

        return CollectionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def statistics(self) -> StatisticsRequestBuilder:
        """
        The statistics property
        """
        from .statistics.statistics_request_builder import StatisticsRequestBuilder

        return StatisticsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def videos(self) -> VideosRequestBuilder:
        """
        The videos property
        """
        from .videos.videos_request_builder import VideosRequestBuilder

        return VideosRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ....models.manage_videos.video import Video
    from ....models.manage_videos.video_create import VideoCreate
    from .fetch.fetch_request_builder import FetchRequestBuilder
    from .get_order_by_query_parameter_type import GetOrderByQueryParameterType
    from .item.with_video_item_request_builder import WithVideoItemRequestBuilder
    from .videos_get_response import VideosGetResponse

class VideosRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /library/{libraryId}/videos
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new VideosRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/library/{libraryId}/videos?itemsPerPage={itemsPerPage}&orderBy={orderBy}&page={page}{&collection,search}", path_parameters)
    
    def by_video_id(self,video_id: str) -> WithVideoItemRequestBuilder:
        """
        Gets an item from the StreamApiClient.library.item.videos.item collection
        param video_id: Unique identifier of the item
        Returns: WithVideoItemRequestBuilder
        """
        if video_id is None:
            raise TypeError("video_id cannot be null.")
        from .item.with_video_item_request_builder import WithVideoItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["videoId"] = video_id
        return WithVideoItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[VideosRequestBuilderGetQueryParameters]] = None) -> Optional[VideosGetResponse]:
        """
        [ListVideos API Docs](https://docs.bunny.net/reference/video_list)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[VideosGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .videos_get_response import VideosGetResponse

        return await self.request_adapter.send_async(request_info, VideosGetResponse, None)
    
    async def post(self,body: VideoCreate, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[Video]:
        """
        [CreateVideo API Docs](https://docs.bunny.net/reference/video_createvideo)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[Video]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ....models.manage_videos.video import Video

        return await self.request_adapter.send_async(request_info, Video, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[VideosRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        [ListVideos API Docs](https://docs.bunny.net/reference/video_list)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: VideoCreate, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [CreateVideo API Docs](https://docs.bunny.net/reference/video_createvideo)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, '{+baseurl}/library/{libraryId}/videos', self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> VideosRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: VideosRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return VideosRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def fetch(self) -> FetchRequestBuilder:
        """
        The fetch property
        """
        from .fetch.fetch_request_builder import FetchRequestBuilder

        return FetchRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class VideosRequestBuilderGetQueryParameters():
        """
        [ListVideos API Docs](https://docs.bunny.net/reference/video_list)
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "items_per_page":
                return "itemsPerPage"
            if original_name == "order_by":
                return "orderBy"
            if original_name == "collection":
                return "collection"
            if original_name == "page":
                return "page"
            if original_name == "search":
                return "search"
            return original_name
        
        collection: Optional[str] = None

        items_per_page: Optional[int] = None

        order_by: Optional[GetOrderByQueryParameterType] = None

        page: Optional[int] = None

        search: Optional[str] = None

    
    @dataclass
    class VideosRequestBuilderGetRequestConfiguration(RequestConfiguration[VideosRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class VideosRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .caption import Caption
    from .chapter import Chapter
    from .meta_tag import MetaTag
    from .moment import Moment
    from .transcoding_message.transcoding_message import TranscodingMessage
    from .video_category import Video_category

@dataclass
class Video(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The available resolutions of the video
    available_resolutions: Optional[str] = None
    # The average watch time of the video in seconds
    average_watch_time: Optional[int] = None
    # The captions property
    captions: Optional[List[Caption]] = None
    # The automatically detected category of the video
    category: Optional[Video_category] = None
    # The list of chapters available for the video
    chapters: Optional[List[Chapter]] = None
    # The ID of the collection where the video belongs
    collection_id: Optional[str] = None
    # The date when the video was uploaded
    date_uploaded: Optional[datetime.datetime] = None
    # The current encode progress of the video
    encode_progress: Optional[int] = None
    # The framerate of the video
    framerate: Optional[float] = None
    # The unique ID of the video
    guid: Optional[str] = None
    # Determines if the video has MP4 fallback files generated
    has_m_p4_fallback: Optional[bool] = None
    # The height of the original video file
    height: Optional[int] = None
    # Determines if the video is publicly accessible
    is_public: Optional[bool] = None
    # The duration of the video in seconds
    length: Optional[int] = None
    # The list of meta tags that have been added to the video
    meta_tags: Optional[List[MetaTag]] = None
    # The list of moments available for the video
    moments: Optional[List[Moment]] = None
    # The rotation of the video
    rotation: Optional[int] = None
    # The status of the video.
    status: Optional[float] = None
    # The amount of storage used by this video
    storage_size: Optional[int] = None
    # The number of thumbnails generated for this video
    thumbnail_count: Optional[int] = None
    # The file name of the thumbnail inside of the storage
    thumbnail_file_name: Optional[str] = None
    # The title of the video
    title: Optional[str] = None
    # The total video watch time in seconds
    total_watch_time: Optional[int] = None
    # The list of transcoding messages that describe potential issues while the video was transcoding
    transcoding_messages: Optional[List[TranscodingMessage]] = None
    # The ID of the video library that the video belongs to
    video_library_id: Optional[int] = None
    # The number of views the video received
    views: Optional[int] = None
    # The width of the original video file
    width: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Video:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Video
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return Video()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .caption import Caption
        from .chapter import Chapter
        from .meta_tag import MetaTag
        from .moment import Moment
        from .transcoding_message.transcoding_message import TranscodingMessage
        from .video_category import Video_category

        from .caption import Caption
        from .chapter import Chapter
        from .meta_tag import MetaTag
        from .moment import Moment
        from .transcoding_message.transcoding_message import TranscodingMessage
        from .video_category import Video_category

        fields: Dict[str, Callable[[Any], None]] = {
            "availableResolutions": lambda n : setattr(self, 'available_resolutions', n.get_str_value()),
            "averageWatchTime": lambda n : setattr(self, 'average_watch_time', n.get_int_value()),
            "captions": lambda n : setattr(self, 'captions', n.get_collection_of_object_values(Caption)),
            "category": lambda n : setattr(self, 'category', n.get_enum_value(Video_category)),
            "chapters": lambda n : setattr(self, 'chapters', n.get_collection_of_object_values(Chapter)),
            "collectionId": lambda n : setattr(self, 'collection_id', n.get_str_value()),
            "dateUploaded": lambda n : setattr(self, 'date_uploaded', n.get_datetime_value()),
            "encodeProgress": lambda n : setattr(self, 'encode_progress', n.get_int_value()),
            "framerate": lambda n : setattr(self, 'framerate', n.get_float_value()),
            "guid": lambda n : setattr(self, 'guid', n.get_str_value()),
            "hasMP4Fallback": lambda n : setattr(self, 'has_m_p4_fallback', n.get_bool_value()),
            "height": lambda n : setattr(self, 'height', n.get_int_value()),
            "isPublic": lambda n : setattr(self, 'is_public', n.get_bool_value()),
            "length": lambda n : setattr(self, 'length', n.get_int_value()),
            "metaTags": lambda n : setattr(self, 'meta_tags', n.get_collection_of_object_values(MetaTag)),
            "moments": lambda n : setattr(self, 'moments', n.get_collection_of_object_values(Moment)),
            "rotation": lambda n : setattr(self, 'rotation', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_float_value()),
            "storageSize": lambda n : setattr(self, 'storage_size', n.get_int_value()),
            "thumbnailCount": lambda n : setattr(self, 'thumbnail_count', n.get_int_value()),
            "thumbnailFileName": lambda n : setattr(self, 'thumbnail_file_name', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "totalWatchTime": lambda n : setattr(self, 'total_watch_time', n.get_int_value()),
            "transcodingMessages": lambda n : setattr(self, 'transcoding_messages', n.get_collection_of_object_values(TranscodingMessage)),
            "videoLibraryId": lambda n : setattr(self, 'video_library_id', n.get_int_value()),
            "views": lambda n : setattr(self, 'views', n.get_int_value()),
            "width": lambda n : setattr(self, 'width', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("captions", self.captions)
        writer.write_collection_of_object_values("chapters", self.chapters)
        writer.write_str_value("collectionId", self.collection_id)
        writer.write_collection_of_object_values("metaTags", self.meta_tags)
        writer.write_collection_of_object_values("moments", self.moments)
        writer.write_str_value("title", self.title)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from ....models.manage_collections.collection import Collection

@dataclass
class CollectionsGetResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The currentPage property
    current_page: Optional[int] = None
    # The items property
    items: Optional[List[Collection]] = None
    # The itemsPerPage property
    items_per_page: Optional[int] = None
    # The totalItems property
    total_items: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CollectionsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CollectionsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CollectionsGetResponse()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from ....models.manage_collections.collection import Collection

        from ....models.manage_collections.collection import Collection

        fields: Dict[str, Callable[[Any], None]] = {
            "currentPage": lambda n : setattr(self, 'current_page', n.get_int_value()),
            "items": lambda n : setattr(self, 'items', n.get_collection_of_object_values(Collection)),
            "itemsPerPage": lambda n : setattr(self, 'items_per_page', n.get_int_value()),
            "totalItems": lambda n : setattr(self, 'total_items', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("currentPage", self.current_page)
        writer.write_collection_of_object_values("items", self.items)
        writer.write_int_value("itemsPerPage", self.items_per_page)
        writer.write_int_value("totalItems", self.total_items)
        writer.write_additional_data_value(self.additional_data)
    




from kiota_abstractions.authentication.api_key_authentication_provider import (
    ApiKeyAuthenticationProvider,
    KeyLocation,
)
from kiota_http.httpx_request_adapter import HttpxRequestAdapter


from .bunny_api_client.bunny_api_client import BunnyApiClient
from .edge_storage_api_client.edge_storage_api_client import EdgeStorageApiClient
from .stream_api_client.stream_api_client import StreamApiClient
from .logging_api_client.logging_api_client import LoggingApiClient


class BunnySdk:
    @staticmethod
    def create_bunny_api_client(access_key: str) -> BunnyApiClient:
        authentication_provider = ApiKeyAuthenticationProvider(
            key_location=KeyLocation.Header,
            api_key=access_key,
            parameter_name="AccessKey",
        )

        request_adapter = HttpxRequestAdapter(authentication_provider)

        return BunnyApiClient(request_adapter)

    @staticmethod
    def create_edge_storage_api_client(
        access_key: str, base_url: str
    ) -> EdgeStorageApiClient:
        authentication_provider = ApiKeyAuthenticationProvider(
            key_location=KeyLocation.Header,
            api_key=access_key,
            parameter_name="AccessKey",
        )

        request_adapter = HttpxRequestAdapter(authentication_provider)
        request_adapter.base_url = base_url

        return EdgeStorageApiClient(request_adapter)

    @staticmethod
    def create_stream_api_client(access_key: str) -> StreamApiClient:
        authentication_provider = ApiKeyAuthenticationProvider(
            key_location=KeyLocation.Header,
            api_key=access_key,
            parameter_name="AccessKey",
        )

        request_adapter = HttpxRequestAdapter(authentication_provider)

        return StreamApiClient(request_adapter)

    @staticmethod
    def create_logging_api_client(access_key: str) -> LoggingApiClient:
        authentication_provider = ApiKeyAuthenticationProvider(
            key_location=KeyLocation.Header,
            api_key=access_key,
            parameter_name="AccessKey",
        )

        request_adapter = HttpxRequestAdapter(authentication_provider)

        return LoggingApiClient(request_adapter)



from enum import Enum

class EdgeReplicationRegions(str, Enum):
    DE = "DE",
    UK = "UK",
    SE = "SE",
    CZ = "CZ",
    ES = "ES",
    NY = "NY",
    LA = "LA",
    WA = "WA",
    MI = "MI",
    SG = "SG",
    HK = "HK",
    JP = "JP",
    SYD = "SYD",
    BR = "BR",
    JH = "JH",




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from ..video import Video

@dataclass
class VideoPlayData(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The allowEarlyPlay property
    allow_early_play: Optional[bool] = None
    # The captionsBackground property
    captions_background: Optional[str] = None
    # The captionsFontColor property
    captions_font_color: Optional[str] = None
    # The captionsFontSize property
    captions_font_size: Optional[int] = None
    # The captionsPath property
    captions_path: Optional[str] = None
    # The controls property
    controls: Optional[str] = None
    # The drmVersion property
    drm_version: Optional[int] = None
    # The enableDRM property
    enable_d_r_m: Optional[bool] = None
    # The enableMP4Fallback property
    enable_m_p4_fallback: Optional[bool] = None
    # The fallbackUrl property
    fallback_url: Optional[str] = None
    # The fontFamily property
    font_family: Optional[str] = None
    # The originalUrl property
    original_url: Optional[str] = None
    # The playbackSpeeds property
    playback_speeds: Optional[str] = None
    # The playerKeyColor property
    player_key_color: Optional[str] = None
    # The previewUrl property
    preview_url: Optional[str] = None
    # The seekPath property
    seek_path: Optional[str] = None
    # The showHeatmap property
    show_heatmap: Optional[bool] = None
    # The thumbnailUrl property
    thumbnail_url: Optional[str] = None
    # The tokenAuthEnabled property
    token_auth_enabled: Optional[bool] = None
    # The uiLanguage property
    ui_language: Optional[str] = None
    # The vastTagUrl property
    vast_tag_url: Optional[str] = None
    # The video property
    video: Optional[Video] = None
    # The videoPlaylistUrl property
    video_playlist_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VideoPlayData:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VideoPlayData
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VideoPlayData()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from ..video import Video

        from ..video import Video

        fields: Dict[str, Callable[[Any], None]] = {
            "allowEarlyPlay": lambda n : setattr(self, 'allow_early_play', n.get_bool_value()),
            "captionsBackground": lambda n : setattr(self, 'captions_background', n.get_str_value()),
            "captionsFontColor": lambda n : setattr(self, 'captions_font_color', n.get_str_value()),
            "captionsFontSize": lambda n : setattr(self, 'captions_font_size', n.get_int_value()),
            "captionsPath": lambda n : setattr(self, 'captions_path', n.get_str_value()),
            "controls": lambda n : setattr(self, 'controls', n.get_str_value()),
            "drmVersion": lambda n : setattr(self, 'drm_version', n.get_int_value()),
            "enableDRM": lambda n : setattr(self, 'enable_d_r_m', n.get_bool_value()),
            "enableMP4Fallback": lambda n : setattr(self, 'enable_m_p4_fallback', n.get_bool_value()),
            "fallbackUrl": lambda n : setattr(self, 'fallback_url', n.get_str_value()),
            "fontFamily": lambda n : setattr(self, 'font_family', n.get_str_value()),
            "originalUrl": lambda n : setattr(self, 'original_url', n.get_str_value()),
            "playbackSpeeds": lambda n : setattr(self, 'playback_speeds', n.get_str_value()),
            "playerKeyColor": lambda n : setattr(self, 'player_key_color', n.get_str_value()),
            "previewUrl": lambda n : setattr(self, 'preview_url', n.get_str_value()),
            "seekPath": lambda n : setattr(self, 'seek_path', n.get_str_value()),
            "showHeatmap": lambda n : setattr(self, 'show_heatmap', n.get_bool_value()),
            "thumbnailUrl": lambda n : setattr(self, 'thumbnail_url', n.get_str_value()),
            "tokenAuthEnabled": lambda n : setattr(self, 'token_auth_enabled', n.get_bool_value()),
            "uiLanguage": lambda n : setattr(self, 'ui_language', n.get_str_value()),
            "vastTagUrl": lambda n : setattr(self, 'vast_tag_url', n.get_str_value()),
            "video": lambda n : setattr(self, 'video', n.get_object_value(Video)),
            "videoPlaylistUrl": lambda n : setattr(self, 'video_playlist_url', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("allowEarlyPlay", self.allow_early_play)
        writer.write_str_value("captionsBackground", self.captions_background)
        writer.write_str_value("captionsFontColor", self.captions_font_color)
        writer.write_int_value("captionsFontSize", self.captions_font_size)
        writer.write_str_value("captionsPath", self.captions_path)
        writer.write_str_value("controls", self.controls)
        writer.write_int_value("drmVersion", self.drm_version)
        writer.write_bool_value("enableDRM", self.enable_d_r_m)
        writer.write_bool_value("enableMP4Fallback", self.enable_m_p4_fallback)
        writer.write_str_value("fallbackUrl", self.fallback_url)
        writer.write_str_value("fontFamily", self.font_family)
        writer.write_str_value("originalUrl", self.original_url)
        writer.write_str_value("playbackSpeeds", self.playback_speeds)
        writer.write_str_value("playerKeyColor", self.player_key_color)
        writer.write_str_value("previewUrl", self.preview_url)
        writer.write_str_value("seekPath", self.seek_path)
        writer.write_bool_value("showHeatmap", self.show_heatmap)
        writer.write_str_value("thumbnailUrl", self.thumbnail_url)
        writer.write_bool_value("tokenAuthEnabled", self.token_auth_enabled)
        writer.write_str_value("uiLanguage", self.ui_language)
        writer.write_str_value("vastTagUrl", self.vast_tag_url)
        writer.write_object_value("video", self.video)
        writer.write_str_value("videoPlaylistUrl", self.video_playlist_url)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .....models.manage_videos.video import Video
    from .....models.manage_videos.video_create import VideoCreate
    from .....models.structured_success_response import StructuredSuccessResponse
    from .captions.captions_request_builder import CaptionsRequestBuilder
    from .heatmap.heatmap_request_builder import HeatmapRequestBuilder
    from .play.play_request_builder import PlayRequestBuilder
    from .reencode.reencode_request_builder import ReencodeRequestBuilder
    from .repackage.repackage_request_builder import RepackageRequestBuilder
    from .thumbnail.thumbnail_request_builder import ThumbnailRequestBuilder
    from .transcribe.transcribe_request_builder import TranscribeRequestBuilder

class WithVideoItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /library/{libraryId}/videos/{videoId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new WithVideoItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/library/{libraryId}/videos/{videoId}{?enabledResolutions}", path_parameters)
    
    async def delete(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[StructuredSuccessResponse]:
        """
        [DeleteVideo API Docs](https://docs.bunny.net/reference/video_deletevideo)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[StructuredSuccessResponse]
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.structured_success_response import StructuredSuccessResponse

        return await self.request_adapter.send_async(request_info, StructuredSuccessResponse, None)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[Video]:
        """
        [GetVideo API Docs](https://docs.bunny.net/reference/video_getvideo)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[Video]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.manage_videos.video import Video

        return await self.request_adapter.send_async(request_info, Video, None)
    
    async def post(self,body: VideoCreate, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[StructuredSuccessResponse]:
        """
        [UpdateVideo API Docs](https://docs.bunny.net/reference/video_updatevideo)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[StructuredSuccessResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.structured_success_response import StructuredSuccessResponse

        return await self.request_adapter.send_async(request_info, StructuredSuccessResponse, None)
    
    async def put(self,body: bytes, request_configuration: Optional[RequestConfiguration[WithVideoItemRequestBuilderPutQueryParameters]] = None) -> Optional[StructuredSuccessResponse]:
        """
        [UploadVideo API Docs](https://docs.bunny.net/reference/video_uploadvideo)
        param body: Binary request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[StructuredSuccessResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_put_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.structured_success_response import StructuredSuccessResponse

        return await self.request_adapter.send_async(request_info, StructuredSuccessResponse, None)
    
    def to_delete_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [DeleteVideo API Docs](https://docs.bunny.net/reference/video_deletevideo)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.DELETE, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [GetVideo API Docs](https://docs.bunny.net/reference/video_getvideo)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: VideoCreate, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [UpdateVideo API Docs](https://docs.bunny.net/reference/video_updatevideo)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def to_put_request_information(self,body: bytes, request_configuration: Optional[RequestConfiguration[WithVideoItemRequestBuilderPutQueryParameters]] = None) -> RequestInformation:
        """
        [UploadVideo API Docs](https://docs.bunny.net/reference/video_uploadvideo)
        param body: Binary request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.PUT, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_stream_content(body, "application/octet-stream")
        return request_info
    
    def with_url(self,raw_url: str) -> WithVideoItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WithVideoItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WithVideoItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def captions(self) -> CaptionsRequestBuilder:
        """
        The captions property
        """
        from .captions.captions_request_builder import CaptionsRequestBuilder

        return CaptionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def heatmap(self) -> HeatmapRequestBuilder:
        """
        The heatmap property
        """
        from .heatmap.heatmap_request_builder import HeatmapRequestBuilder

        return HeatmapRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def play(self) -> PlayRequestBuilder:
        """
        The play property
        """
        from .play.play_request_builder import PlayRequestBuilder

        return PlayRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def reencode(self) -> ReencodeRequestBuilder:
        """
        The reencode property
        """
        from .reencode.reencode_request_builder import ReencodeRequestBuilder

        return ReencodeRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def repackage(self) -> RepackageRequestBuilder:
        """
        The repackage property
        """
        from .repackage.repackage_request_builder import RepackageRequestBuilder

        return RepackageRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def thumbnail(self) -> ThumbnailRequestBuilder:
        """
        The thumbnail property
        """
        from .thumbnail.thumbnail_request_builder import ThumbnailRequestBuilder

        return ThumbnailRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def transcribe(self) -> TranscribeRequestBuilder:
        """
        The transcribe property
        """
        from .transcribe.transcribe_request_builder import TranscribeRequestBuilder

        return TranscribeRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class WithVideoItemRequestBuilderDeleteRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithVideoItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithVideoItemRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithVideoItemRequestBuilderPutQueryParameters():
        """
        [UploadVideo API Docs](https://docs.bunny.net/reference/video_uploadvideo)
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "enabled_resolutions":
                return "enabledResolutions"
            return original_name
        
        enabled_resolutions: Optional[str] = None

    
    @dataclass
    class WithVideoItemRequestBuilderPutRequestConfiguration(RequestConfiguration[WithVideoItemRequestBuilderPutQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class OEmbedGetResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The height property
    height: Optional[int] = None
    # The html property
    html: Optional[str] = None
    # The providerName property
    provider_name: Optional[str] = None
    # The providerUrl property
    provider_url: Optional[str] = None
    # The thumbnailUrl property
    thumbnail_url: Optional[str] = None
    # The title property
    title: Optional[str] = None
    # The type property
    type: Optional[str] = None
    # The version property
    version: Optional[str] = None
    # The width property
    width: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> OEmbedGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: OEmbedGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return OEmbedGetResponse()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "height": lambda n : setattr(self, 'height', n.get_int_value()),
            "html": lambda n : setattr(self, 'html', n.get_str_value()),
            "providerName": lambda n : setattr(self, 'provider_name', n.get_str_value()),
            "providerUrl": lambda n : setattr(self, 'provider_url', n.get_str_value()),
            "thumbnailUrl": lambda n : setattr(self, 'thumbnail_url', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_str_value()),
            "width": lambda n : setattr(self, 'width', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("height", self.height)
        writer.write_str_value("html", self.html)
        writer.write_str_value("providerName", self.provider_name)
        writer.write_str_value("providerUrl", self.provider_url)
        writer.write_str_value("thumbnailUrl", self.thumbnail_url)
        writer.write_str_value("title", self.title)
        writer.write_str_value("type", self.type)
        writer.write_str_value("version", self.version)
        writer.write_int_value("width", self.width)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class Region(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The AllowLatencyRouting property
    allow_latency_routing: Optional[bool] = None
    # The ContinentCode property
    continent_code: Optional[str] = None
    # The CountryCode property
    country_code: Optional[str] = None
    # The Id property
    id: Optional[int] = None
    # The Latitude property
    latitude: Optional[float] = None
    # The Longitude property
    longitude: Optional[float] = None
    # The Name property
    name: Optional[str] = None
    # The PricePerGigabyte property
    price_per_gigabyte: Optional[float] = None
    # The RegionCode property
    region_code: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Region:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Region
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return Region()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "AllowLatencyRouting": lambda n : setattr(self, 'allow_latency_routing', n.get_bool_value()),
            "ContinentCode": lambda n : setattr(self, 'continent_code', n.get_str_value()),
            "CountryCode": lambda n : setattr(self, 'country_code', n.get_str_value()),
            "Id": lambda n : setattr(self, 'id', n.get_int_value()),
            "Latitude": lambda n : setattr(self, 'latitude', n.get_float_value()),
            "Longitude": lambda n : setattr(self, 'longitude', n.get_float_value()),
            "Name": lambda n : setattr(self, 'name', n.get_str_value()),
            "PricePerGigabyte": lambda n : setattr(self, 'price_per_gigabyte', n.get_float_value()),
            "RegionCode": lambda n : setattr(self, 'region_code', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("AllowLatencyRouting", self.allow_latency_routing)
        writer.write_str_value("ContinentCode", self.continent_code)
        writer.write_str_value("CountryCode", self.country_code)
        writer.write_int_value("Id", self.id)
        writer.write_float_value("Latitude", self.latitude)
        writer.write_float_value("Longitude", self.longitude)
        writer.write_str_value("Name", self.name)
        writer.write_float_value("PricePerGigabyte", self.price_per_gigabyte)
        writer.write_str_value("RegionCode", self.region_code)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from kiota_abstractions.api_client_builder import enable_backing_store_for_serialization_writer_factory, register_default_deserializer, register_default_serializer
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.serialization import ParseNodeFactoryRegistry, SerializationWriterFactoryRegistry
from kiota_serialization_form.form_parse_node_factory import FormParseNodeFactory
from kiota_serialization_form.form_serialization_writer_factory import FormSerializationWriterFactory
from kiota_serialization_json.json_parse_node_factory import JsonParseNodeFactory
from kiota_serialization_json.json_serialization_writer_factory import JsonSerializationWriterFactory
from kiota_serialization_multipart.multipart_serialization_writer_factory import MultipartSerializationWriterFactory
from kiota_serialization_text.text_parse_node_factory import TextParseNodeFactory
from kiota_serialization_text.text_serialization_writer_factory import TextSerializationWriterFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .abusecase.abusecase_request_builder import AbusecaseRequestBuilder
    from .apikey.apikey_request_builder import ApikeyRequestBuilder
    from .compute.compute_request_builder import ComputeRequestBuilder
    from .country.country_request_builder import CountryRequestBuilder
    from .dmca.dmca_request_builder import DmcaRequestBuilder
    from .dnszone.dnszone_request_builder import DnszoneRequestBuilder
    from .pullzone.pullzone_request_builder import PullzoneRequestBuilder
    from .purge.purge_request_builder import PurgeRequestBuilder
    from .region.region_request_builder import RegionRequestBuilder
    from .statistics.statistics_request_builder import StatisticsRequestBuilder
    from .storagezone.storagezone_request_builder import StoragezoneRequestBuilder
    from .user.user_request_builder import UserRequestBuilder
    from .videolibrary.videolibrary_request_builder import VideolibraryRequestBuilder

class BunnyApiClient(BaseRequestBuilder):
    """
    The main entry point of the SDK, exposes the configuration and the fluent API.
    """
    def __init__(self,request_adapter: RequestAdapter) -> None:
        """
        Instantiates a new BunnyApiClient and sets the default values.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        if request_adapter is None:
            raise TypeError("request_adapter cannot be null.")
        super().__init__(request_adapter, "{+baseurl}", None)
        register_default_serializer(JsonSerializationWriterFactory)
        register_default_serializer(TextSerializationWriterFactory)
        register_default_serializer(FormSerializationWriterFactory)
        register_default_serializer(MultipartSerializationWriterFactory)
        register_default_deserializer(JsonParseNodeFactory)
        register_default_deserializer(TextParseNodeFactory)
        register_default_deserializer(FormParseNodeFactory)
        if not self.request_adapter.base_url:
            self.request_adapter.base_url = "https://api.bunny.net"
        self.path_parameters["base_url"] = self.request_adapter.base_url
    
    @property
    def abusecase(self) -> AbusecaseRequestBuilder:
        """
        The abusecase property
        """
        from .abusecase.abusecase_request_builder import AbusecaseRequestBuilder

        return AbusecaseRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def apikey(self) -> ApikeyRequestBuilder:
        """
        The apikey property
        """
        from .apikey.apikey_request_builder import ApikeyRequestBuilder

        return ApikeyRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def compute(self) -> ComputeRequestBuilder:
        """
        The compute property
        """
        from .compute.compute_request_builder import ComputeRequestBuilder

        return ComputeRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def country(self) -> CountryRequestBuilder:
        """
        The country property
        """
        from .country.country_request_builder import CountryRequestBuilder

        return CountryRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def dmca(self) -> DmcaRequestBuilder:
        """
        The dmca property
        """
        from .dmca.dmca_request_builder import DmcaRequestBuilder

        return DmcaRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def dnszone(self) -> DnszoneRequestBuilder:
        """
        The dnszone property
        """
        from .dnszone.dnszone_request_builder import DnszoneRequestBuilder

        return DnszoneRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def pullzone(self) -> PullzoneRequestBuilder:
        """
        The pullzone property
        """
        from .pullzone.pullzone_request_builder import PullzoneRequestBuilder

        return PullzoneRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def purge(self) -> PurgeRequestBuilder:
        """
        The purge property
        """
        from .purge.purge_request_builder import PurgeRequestBuilder

        return PurgeRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def region(self) -> RegionRequestBuilder:
        """
        The region property
        """
        from .region.region_request_builder import RegionRequestBuilder

        return RegionRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def statistics(self) -> StatisticsRequestBuilder:
        """
        The statistics property
        """
        from .statistics.statistics_request_builder import StatisticsRequestBuilder

        return StatisticsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def storagezone(self) -> StoragezoneRequestBuilder:
        """
        The storagezone property
        """
        from .storagezone.storagezone_request_builder import StoragezoneRequestBuilder

        return StoragezoneRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def user(self) -> UserRequestBuilder:
        """
        The user property
        """
        from .user.user_request_builder import UserRequestBuilder

        return UserRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def videolibrary(self) -> VideolibraryRequestBuilder:
        """
        The videolibrary property
        """
        from .videolibrary.videolibrary_request_builder import VideolibraryRequestBuilder

        return VideolibraryRequestBuilder(self.request_adapter, self.path_parameters)
    




from enum import Enum

class Properties_crop_gravity(str, Enum):
    Center = "center",
    Forget = "forget",
    East = "east",
    North = "north",
    South = "south",
    West = "west",
    Northeast = "northeast",
    Northwest = "northwest",
    Southeast = "southeast",
    Southwest = "southwest",




from __future__ import annotations
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .add_or_update.add_or_update_request_builder import AddOrUpdateRequestBuilder
    from .item.with_edge_rule_item_request_builder import WithEdgeRuleItemRequestBuilder

class EdgerulesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /pullzone/{-id}/edgerules
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new EdgerulesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/pullzone/{%2Did}/edgerules", path_parameters)
    
    def by_edge_rule_id(self,edge_rule_id: str) -> WithEdgeRuleItemRequestBuilder:
        """
        Gets an item from the BunnyApiClient.pullzone.item.edgerules.item collection
        param edge_rule_id: The ID of the Edge Rule that should be deleted
        Returns: WithEdgeRuleItemRequestBuilder
        """
        if edge_rule_id is None:
            raise TypeError("edge_rule_id cannot be null.")
        from .item.with_edge_rule_item_request_builder import WithEdgeRuleItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["edgeRuleId"] = edge_rule_id
        return WithEdgeRuleItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    @property
    def add_or_update(self) -> AddOrUpdateRequestBuilder:
        """
        The addOrUpdate property
        """
        from .add_or_update.add_or_update_request_builder import AddOrUpdateRequestBuilder

        return AddOrUpdateRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class Collection(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The unique ID of the collection
    guid: Optional[str] = None
    # The name of the collection
    name: Optional[str] = None
    # The URLs of preview images of videos in the collection
    preview_image_urls: Optional[str] = None
    # The IDs of videos to be used as preview icons
    preview_video_ids: Optional[str] = None
    # The total storage size of the collection
    total_size: Optional[int] = None
    # The number of videos that the collection contains
    video_count: Optional[int] = None
    # The video library ID that contains the collection
    video_library_id: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Collection:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Collection
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return Collection()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "guid": lambda n : setattr(self, 'guid', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "previewImageUrls": lambda n : setattr(self, 'preview_image_urls', n.get_str_value()),
            "previewVideoIds": lambda n : setattr(self, 'preview_video_ids', n.get_str_value()),
            "totalSize": lambda n : setattr(self, 'total_size', n.get_int_value()),
            "videoCount": lambda n : setattr(self, 'video_count', n.get_int_value()),
            "videoLibraryId": lambda n : setattr(self, 'video_library_id', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("name", self.name)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class Connection(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The ConnectionType property
    connection_type: Optional[float] = None
    # The main custom connected CDN domain
    main_custom_domain: Optional[str] = None
    # The total amount of bandwidth served by this zone this month
    monthly_bandwidth_used: Optional[float] = None
    # The total monthly charges incurred by this zone
    monthly_charges: Optional[float] = None
    # The ID of the connected pull zone
    pull_zone_id: Optional[int] = None
    # The name of the connected pull zone
    pull_zone_name: Optional[str] = None
    # The Tier property
    tier: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Connection:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Connection
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return Connection()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "ConnectionType": lambda n : setattr(self, 'connection_type', n.get_float_value()),
            "MainCustomDomain": lambda n : setattr(self, 'main_custom_domain', n.get_str_value()),
            "MonthlyBandwidthUsed": lambda n : setattr(self, 'monthly_bandwidth_used', n.get_float_value()),
            "MonthlyCharges": lambda n : setattr(self, 'monthly_charges', n.get_float_value()),
            "PullZoneId": lambda n : setattr(self, 'pull_zone_id', n.get_int_value()),
            "PullZoneName": lambda n : setattr(self, 'pull_zone_name', n.get_str_value()),
            "Tier": lambda n : setattr(self, 'tier', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("ConnectionType", self.connection_type)
        writer.write_str_value("MainCustomDomain", self.main_custom_domain)
        writer.write_float_value("MonthlyBandwidthUsed", self.monthly_bandwidth_used)
        writer.write_float_value("MonthlyCharges", self.monthly_charges)
        writer.write_int_value("PullZoneId", self.pull_zone_id)
        writer.write_str_value("PullZoneName", self.pull_zone_name)
        writer.write_float_value("Tier", self.tier)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .add.add_request_builder import AddRequestBuilder
    from .item.with_variable_item_request_builder import WithVariableItemRequestBuilder

class VariablesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /compute/script/{id}/variables
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new VariablesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/compute/script/{id}/variables", path_parameters)
    
    def by_variable_id(self,variable_id: int) -> WithVariableItemRequestBuilder:
        """
        Gets an item from the BunnyApiClient.compute.script.item.variables.item collection
        param variable_id: The ID of the Environment Variable that will be updated
        Returns: WithVariableItemRequestBuilder
        """
        if variable_id is None:
            raise TypeError("variable_id cannot be null.")
        from .item.with_variable_item_request_builder import WithVariableItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["variableId"] = variable_id
        return WithVariableItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    @property
    def add(self) -> AddRequestBuilder:
        """
        The add property
        """
        from .add.add_request_builder import AddRequestBuilder

        return AddRequestBuilder(self.request_adapter, self.path_parameters)
    

